using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Runtime;
using System.Runtime.InteropServices;

namespace RSSReader.Controls
{

	#region Listview column sort utility class 

	public class ListViewSortHelper : IComparer
	{

		#region Public events 

		public event EventHandler	BeforeSort;
		public event EventHandler	AfterSort;

		#endregion 

		#region Private variables 

		/// <summary>
		/// Specifies the column to be sorted
		/// </summary>
		private int ColumnToSort = 0;

		/// <summary>
		/// Specifies the order in which to sort (i.e. 'Ascending').
		/// </summary>
		private SortOrder OrderOfSort = SortOrder.Ascending;

		/// <summary>
		/// The list of custom comparers
		/// </summary>
		private Hashtable customComparers;

		private ListView view;

		#endregion 

		#region Constructor 

		private ListViewSortHelper() {} 

		/// <summary>
		/// Class constructor.  Initializes various elements
		/// </summary>
		public ListViewSortHelper( ListView view )
		{

			// Initialize the column to '0'
			ColumnToSort = 0;

			// Initialize the sort order to 'none'
			OrderOfSort = SortOrder.None;

			// Initialize the list of custom comparers
			customComparers = new Hashtable();

			this.view = view;
			view.ListViewItemSorter = this;
			view.ColumnClick += new ColumnClickEventHandler(view_ColumnClick);

		}

		#endregion 

		#region IComparer implementation 

		private IComparer GetComparer( int columnIndex )
		{

			if( !customComparers.ContainsKey( columnIndex ) || customComparers[ columnIndex ] == null )
				return new CaseInsensitiveComparer();

			return (IComparer) customComparers[ columnIndex ];

		}

		/// <summary>
		/// This method is inherited from the IComparer interface.  It compares the two objects passed using a case insensitive comparison.
		/// </summary>
		/// <param name="x">First object to be compared</param>
		/// <param name="y">Second object to be compared</param>
		/// <returns>The result of the comparison. "0" if equal, negative if 'x' is less than 'y' and positive if 'x' is greater than 'y'</returns>
		public int Compare(object x, object y)
		{

			// Cast the objects to be compared to ListViewItem objects
			ListViewItem lviLeft = (ListViewItem) x;
			ListViewItem lviRight = (ListViewItem) y;

			if( (lviLeft is ThreadListItem) && (lviRight is ThreadListItem) )
			{

				// Get the indent level of the items 
				int indentLeft = ((ThreadListItem) lviLeft).IndentLevel;
				int indentRight = ((ThreadListItem) lviRight).IndentLevel;

				// Since this method is called right after inserting a new ListItem,
				// check the SubItems.Count property to see if we should event bother
				// sorting these two items.  A newly-added ListViewItem will probably
				// not have the full complement of columns yet.
				if( lviLeft.SubItems.Count <= SortColumn || lviRight.SubItems.Count <= SortColumn )
					return 0;

				// If IndentLevel does not match, cannot compare these two items
				if( indentLeft != indentRight )
					return 0;

				// Do not sort indented items
				if( indentLeft > 0 || indentRight > 0 )
					return 0;

			}

			// Get the comparer for the column
			IComparer comparer = GetComparer( ColumnToSort );

			// Compare the two items
			int compareResult = comparer.Compare( lviLeft.SubItems[ColumnToSort].Text, lviRight.SubItems[ColumnToSort].Text );

			// Calculate correct return value based on object comparison
			if( OrderOfSort == SortOrder.Ascending)
			{
				// Ascending sort is selected, return normal result of compare operation
				return compareResult;
			}
			else if( OrderOfSort == SortOrder.Descending)
			{
				// Descending sort is selected, return negative result of compare operation
				return (-compareResult);
			}
			else
			{
				// Return '0' to indicate they are equal
				return 0;
			}

		}

		#endregion 

		#region Public properties 

		/// <summary>
		/// Gets or sets the number of the column to which to apply the sorting operation (Defaults to '0').
		/// </summary>
		public int SortColumn
		{
			set { ColumnToSort = value; }
			get { return ColumnToSort; }
		}

		/// <summary>
		/// Gets or sets the order of sorting to apply (for example, 'Ascending' or 'Descending').
		/// </summary>
		public SortOrder Order
		{
			set { OrderOfSort = value; }
			get { return OrderOfSort; }
		}

		#endregion

		#region Public methods 

		public void SetColumnComparer( int ColumnIndex, IComparer Comparer )
		{

			this.customComparers[ ColumnIndex ] = Comparer;

		}

		/// <summary>
		/// Sorts the ListView contents based on the column index
		/// </summary>
		public void SortOnColumn( int column )
		{
			view_ColumnClick( view, new ColumnClickEventArgs( column ) );
		}

		#endregion 

		#region Listview event handlers 

		private void view_ColumnClick(object sender, ColumnClickEventArgs e)
		{

			if( BeforeSort != null )
				BeforeSort( sender, EventArgs.Empty );
			
			// Determine if clicked column is already the column that is being sorted.
			if(  e.Column == SortColumn )
			{

				// Reverse the current sort direction for this column.
				if( Order == SortOrder.Ascending)
					Order = SortOrder.Descending;
				else
					Order = SortOrder.Ascending;

			}
			else
			{

				// Set the column number that is to be sorted; default to ascending.
				SortColumn = e.Column;
				Order = SortOrder.Ascending;


			}

			// Draw the appropriate sort indicator
			ListViewHelper.ShowSortIndicator( view, SortColumn, Order );

			// Perform the sort with these new sort options.
			this.view.Sort();

			if( view.SelectedItems.Count > 0 )
				view.SelectedItems[0].EnsureVisible();

			if( AfterSort != null )
				AfterSort( sender, EventArgs.Empty );
		
		}

		#endregion 

	}

	#endregion 

	#region ListViewHelper class 

	internal class ListViewHelper
	{

		#region Win32 API constants and enumerations 

		public enum HeaderIconFlags
		{
			HDI_BITMAP			= 0x10,
			HDI_IMAGE			= 0x20,
			HDI_FORMAT			= 0x04,
			HDI_TEXT			= 0x02
		}

		public enum HeaderFormatFlags
		{
			HDF_BITMAP_ON_RIGHT		= 0x1000,
			HDF_BITMAP				= 0x2000,
			HDF_IMAGE				= 0x800,
			HDF_STRING				= 0x4000
		}

		public enum ListViewStyleExtended 
		{
			LVS_EX_GRIDLINES        =0x00000001,
			LVS_EX_SUBITEMIMAGES    =0x00000002,
			LVS_EX_CHECKBOXES       =0x00000004,
			LVS_EX_TRACKSELECT      =0x00000008,
			LVS_EX_HEADERDRAGDROP   =0x00000010,
			LVS_EX_FULLROWSELECT    =0x00000020, 
			LVS_EX_ONECLICKACTIVATE =0x00000040,
			LVS_EX_TWOCLICKACTIVATE =0x00000080,
			LVS_EX_FLATSB           =0x00000100,
			LVS_EX_REGIONAL         =0x00000200,
			LVS_EX_INFOTIP          =0x00000400,
			LVS_EX_UNDERLINEHOT     =0x00000800,
			LVS_EX_UNDERLINECOLD    =0x00001000,
			LVS_EX_MULTIWORKAREAS   =0x00002000,
			LVS_EX_LABELTIP         =0x00004000,
			LVS_EX_BORDERSELECT     =0x00008000, 
			LVS_EX_DOUBLEBUFFER     =0x00010000,
			LVS_EX_HIDELABELS       =0x00020000,
			LVS_EX_SINGLEROW        =0x00040000,
			LVS_EX_SNAPTOGRID       =0x00080000,  
			LVS_EX_SIMPLESELECT     =0x00100000  
		}

		public enum ListViewItemFlags 
		{
			LVIF_TEXT               = 0x0001,
			LVIF_IMAGE              = 0x0002,
			LVIF_PARAM              = 0x0004,
			LVIF_STATE              = 0x0008,
			LVIF_INDENT             = 0x0010,
			LVIF_NORECOMPUTE        = 0x0800
		}

		public enum ListViewMessage : int
		{
			LVM_FIRST                    = 0x1000,
			LVM_GETBKCOLOR               = ( LVM_FIRST + 0 ),
			LVM_SETBKCOLOR               = ( LVM_FIRST + 1 ),
			LVM_GETIMAGELIST             = ( LVM_FIRST + 2 ),
			LVM_SETIMAGELIST             = ( LVM_FIRST + 3 ),
			LVM_GETITEMCOUNT             = ( LVM_FIRST + 4 ),
			LVM_GETITEMA                 = ( LVM_FIRST + 5 ),
			LVM_GETITEMW                 = ( LVM_FIRST + 75 ),
			LVM_SETITEMA                 = ( LVM_FIRST + 6 ),
			LVM_SETITEMW                 = ( LVM_FIRST + 76 ),
			LVM_INSERTITEMA              = ( LVM_FIRST + 7 ),
			LVM_INSERTITEMW              = ( LVM_FIRST + 77 ),
			LVM_DELETEITEM               = ( LVM_FIRST + 8 ),
			LVM_DELETEALLITEMS           = ( LVM_FIRST + 9 ),
			LVM_GETCALLBACKMASK          = ( LVM_FIRST + 10 ),
			LVM_SETCALLBACKMASK          = ( LVM_FIRST + 11 ),
			LVM_GETNEXTITEM              = ( LVM_FIRST + 12 ),
			LVM_FINDITEMA                = ( LVM_FIRST + 13 ),
			LVM_FINDITEMW                = ( LVM_FIRST + 83 ),
			LVM_GETITEMRECT              = ( LVM_FIRST + 14 ),
			LVM_SETITEMPOSITION          = ( LVM_FIRST + 15 ),
			LVM_GETITEMPOSITION          = ( LVM_FIRST + 16 ),
			LVM_GETSTRINGWIDTHA          = ( LVM_FIRST + 17 ),
			LVM_GETSTRINGWIDTHW          = ( LVM_FIRST + 87 ),
			LVM_HITTEST                  = ( LVM_FIRST + 18 ),
			LVM_ENSUREVISIBLE            = ( LVM_FIRST + 19 ),
			LVM_SCROLL                   = ( LVM_FIRST + 20 ),
			LVM_REDRAWITEMS              = ( LVM_FIRST + 21 ),
			LVM_ARRANGE                  = ( LVM_FIRST + 22 ),
			LVM_EDITLABELA               = ( LVM_FIRST + 23 ),
			LVM_EDITLABELW               = ( LVM_FIRST + 118 ),
			LVM_GETEDITCONTROL           = ( LVM_FIRST + 24 ),
			LVM_GETCOLUMNA               = ( LVM_FIRST + 25 ),
			LVM_GETCOLUMNW               = ( LVM_FIRST + 95 ),
			LVM_SETCOLUMNA               = ( LVM_FIRST + 26 ),
			LVM_SETCOLUMNW               = ( LVM_FIRST + 96 ),
			LVM_INSERTCOLUMNA            = ( LVM_FIRST + 27 ),
			LVM_INSERTCOLUMNW            = ( LVM_FIRST + 97 ),
			LVM_DELETECOLUMN             = ( LVM_FIRST + 28 ),
			LVM_GETCOLUMNWIDTH           = ( LVM_FIRST + 29 ),
			LVM_SETCOLUMNWIDTH           = ( LVM_FIRST + 30 ),
			LVM_GETHEADER                = ( LVM_FIRST + 31 ),
			LVM_CREATEDRAGIMAGE          = ( LVM_FIRST + 33 ),
			LVM_GETVIEWRECT              = ( LVM_FIRST + 34 ),
			LVM_GETTEXTCOLOR             = ( LVM_FIRST + 35 ),
			LVM_SETTEXTCOLOR             = ( LVM_FIRST + 36 ),
			LVM_GETTEXTBKCOLOR           = ( LVM_FIRST + 37 ),
			LVM_SETTEXTBKCOLOR           = ( LVM_FIRST + 38 ),
			LVM_GETTOPINDEX              = ( LVM_FIRST + 39 ),
			LVM_GETCOUNTPERPAGE          = ( LVM_FIRST + 40 ),
			LVM_GETORIGIN                = ( LVM_FIRST + 41 ),
			LVM_UPDATE                   = ( LVM_FIRST + 42 ),
			LVM_SETITEMSTATE             = ( LVM_FIRST + 43 ),
			LVM_GETITEMSTATE             = ( LVM_FIRST + 44 ),
			LVM_GETITEMTEXTA             = ( LVM_FIRST + 45 ),
			LVM_GETITEMTEXTW             = ( LVM_FIRST + 115 ),
			LVM_SETITEMTEXTA             = ( LVM_FIRST + 46 ),
			LVM_SETITEMTEXTW             = ( LVM_FIRST + 116 ),
			LVM_SETITEMCOUNT             = ( LVM_FIRST + 47 ),
			LVM_SORTITEMS                = ( LVM_FIRST + 48 ),
			LVM_SETITEMPOSITION32        = ( LVM_FIRST + 49 ),
			LVM_GETSELECTEDCOUNT         = ( LVM_FIRST + 50 ),
			LVM_GETITEMSPACING           = ( LVM_FIRST + 51 ),
			LVM_GETISEARCHSTRINGA        = ( LVM_FIRST + 52 ),
			LVM_GETISEARCHSTRINGW        = ( LVM_FIRST + 117 ),
			LVM_SETICONSPACING           = ( LVM_FIRST + 53 ),
			LVM_SETEXTENDEDLISTVIEWSTYLE = ( LVM_FIRST + 54 ),
			LVM_GETEXTENDEDLISTVIEWSTYLE = ( LVM_FIRST + 55 ),
			LVM_GETSUBITEMRECT           = ( LVM_FIRST + 56 ),
			LVM_SUBITEMHITTEST           = ( LVM_FIRST + 57 ),
			LVM_SETCOLUMNORDERARRAY      = ( LVM_FIRST + 58 ),
			LVM_GETCOLUMNORDERARRAY      = ( LVM_FIRST + 59 ),
			LVM_SETHOTITEM               = ( LVM_FIRST + 60 ),
			LVM_GETHOTITEM               = ( LVM_FIRST + 61 ),
			LVM_SETHOTCURSOR             = ( LVM_FIRST + 62 ),
			LVM_GETHOTCURSOR             = ( LVM_FIRST + 63 ),
			LVM_APPROXIMATEVIEWRECT      = ( LVM_FIRST + 64 ),
			LVM_SETWORKAREAS             = ( LVM_FIRST + 65 ),
			LVM_GETWORKAREAS             = ( LVM_FIRST + 70 ),
			LVM_GETNUMBEROFWORKAREAS     = ( LVM_FIRST + 73 ),
			LVM_GETSELECTIONMARK         = ( LVM_FIRST + 66 ),
			LVM_SETSELECTIONMARK         = ( LVM_FIRST + 67 ),
			LVM_SETHOVERTIME             = ( LVM_FIRST + 71 ),
			LVM_GETHOVERTIME             = ( LVM_FIRST + 72 ),
			LVM_SETTOOLTIPS              = ( LVM_FIRST + 74 ),
			LVM_GETTOOLTIPS              = ( LVM_FIRST + 78 ),
			LVM_SORTITEMSEX              = ( LVM_FIRST + 81 ),
			LVM_SETBKIMAGEA              = ( LVM_FIRST + 68 ),
			LVM_SETBKIMAGEW              = ( LVM_FIRST + 138 ),
			LVM_GETBKIMAGEA              = ( LVM_FIRST + 69 ),
			LVM_GETBKIMAGEW              = ( LVM_FIRST + 139 ),
			LVM_SETSELECTEDCOLUMN        = ( LVM_FIRST + 140 ),
			LVM_SETTILEWIDTH             = ( LVM_FIRST + 141 ),
			LVM_SETVIEW                  = ( LVM_FIRST + 142 ),
			LVM_GETVIEW                  = ( LVM_FIRST + 143 ),
			LVM_INSERTGROUP              = ( LVM_FIRST + 145 ),
			LVM_SETGROUPINFO             = ( LVM_FIRST + 147 ),
			LVM_GETGROUPINFO             = ( LVM_FIRST + 149 ),
			LVM_REMOVEGROUP              = ( LVM_FIRST + 150 ),
			LVM_MOVEGROUP                = ( LVM_FIRST + 151 ),
			LVM_MOVEITEMTOGROUP          = ( LVM_FIRST + 154 ),
			LVM_SETGROUPMETRICS          = ( LVM_FIRST + 155 ),
			LVM_GETGROUPMETRICS          = ( LVM_FIRST + 156 ),
			LVM_ENABLEGROUPVIEW          = ( LVM_FIRST + 157 ),
			LVM_SORTGROUPS               = ( LVM_FIRST + 158 ),
			LVM_INSERTGROUPSORTED        = ( LVM_FIRST + 159 ),
			LVM_REMOVEALLGROUPS          = ( LVM_FIRST + 160 ),
			LVM_HASGROUP                 = ( LVM_FIRST + 161 ),
			LVM_SETTILEVIEWINFO          = ( LVM_FIRST + 162 ),
			LVM_GETTILEVIEWINFO          = ( LVM_FIRST + 163 ),
			LVM_SETTILEINFO              = ( LVM_FIRST + 164 ),
			LVM_GETTILEINFO              = ( LVM_FIRST + 165 ),
			LVM_SETINSERTMARK            = ( LVM_FIRST + 166 ),
			LVM_GETINSERTMARK            = ( LVM_FIRST + 167 ),
			LVM_INSERTMARKHITTEST        = ( LVM_FIRST + 168 ),
			LVM_GETINSERTMARKRECT        = ( LVM_FIRST + 169 ),
			LVM_SETINSERTMARKCOLOR       = ( LVM_FIRST + 170 ),
			LVM_GETINSERTMARKCOLOR       = ( LVM_FIRST + 171 ),
			LVM_SETINFOTIP               = ( LVM_FIRST + 173 ),
			LVM_GETSELECTEDCOLUMN        = ( LVM_FIRST + 174 ),
			LVM_ISGROUPVIEWENABLED       = ( LVM_FIRST + 175 ),
			LVM_GETOUTLINECOLOR          = ( LVM_FIRST + 176 ),
			LVM_SETOUTLINECOLOR          = ( LVM_FIRST + 177 ),
			LVM_CANCELEDITLABEL          = ( LVM_FIRST + 179 ),
			LVM_MAPINDEXTOID             = ( LVM_FIRST + 180 ),
			LVM_MAPIDTOINDEX             = ( LVM_FIRST + 181 )                                
		}

		public enum ListViewNotification : int
		{
			LVN_FIRST           = 0x0000 - 100,
			LVN_ITEMCHANGING    = ( LVN_FIRST-0 ),
			LVN_ITEMCHANGED     = ( LVN_FIRST-1 ),
			LVN_INSERTITEM      = ( LVN_FIRST-2 ),
			LVN_DELETEITEM      = ( LVN_FIRST-3 ),
			LVN_DELETEALLITEMS  = ( LVN_FIRST-4 ),
			LVN_BEGINLABELEDITA = ( LVN_FIRST-5 ),
			LVN_ENDLABELEDITA   = ( LVN_FIRST-6 ),
			LVN_COLUMNCLICK     = ( LVN_FIRST-8 ),
			LVN_BEGINDRAG       = ( LVN_FIRST-9 ),
			LVN_BEGINRDRAG      = ( LVN_FIRST-11 ),
			LVN_ODCACHEHINT     = ( LVN_FIRST-13 ),
			LVN_ITEMACTIVATE    = ( LVN_FIRST-14 ),
			LVN_ODSTATECHANGED  = ( LVN_FIRST-15 ),
			LVN_HOTTRACK        = ( LVN_FIRST-21 ),
			LVN_GETDISPINFOA    = ( LVN_FIRST-50 ),
			LVN_SETDISPINFOA    = ( LVN_FIRST-51 ),
			LVN_ODFINDITEMA     = ( LVN_FIRST-52 ),
			LVN_KEYDOWN         = ( LVN_FIRST-55 ),
			LVN_MARQUEEBEGIN    = ( LVN_FIRST-56 ),
			LVN_GETINFOTIPA     = ( LVN_FIRST-57 ),
			LVN_GETINFOTIPW     = ( LVN_FIRST-58 ),
			LVN_BEGINLABELEDITW = ( LVN_FIRST-75 ),
			LVN_ENDLABELEDITW   = ( LVN_FIRST-76 ),
			LVN_GETDISPINFOW    = ( LVN_FIRST-77 ),
			LVN_SETDISPINFOW    = ( LVN_FIRST-78 ),
			LVN_ODFINDITEMW     = ( LVN_FIRST-79 ),
			LVN_BEGINSCROLL     = ( LVN_FIRST-80 ),          
			LVN_ENDSCROLL       = ( LVN_FIRST-81 )
		}

		public enum ListViewHitTestFlags: int
		{
			LVHT_NOWHERE = 0x1,
			LVHT_ONITEMICON = 0x2,
			LVHT_ONITEMLABEL = 0x4,
			LVHT_ONITEMSTATEICON = 0x8,
			LVHT_ONITEM = (LVHT_ONITEMICON | LVHT_ONITEMLABEL | LVHT_ONITEMSTATEICON)
		}

		public enum ListViewHeaderMessage: int 
		{
			HDM_FIRST			= 0x1200,
			HDM_SETITEM			= (HDM_FIRST + 4),
			HDM_SETIMAGELIST	= (HDM_FIRST + 8),
			HDM_GETIMAGELIST	= (HDM_FIRST + 9),
			HDM_GETITEM			= (HDM_FIRST + 11),
			HDM_SETITEMW			= (HDM_FIRST + 12)
		}

		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
		public struct LVITEM 
		{
			public	ListViewItemFlags mask;
			public	int iItem;
			public	int iSubItem;
			public	uint state;
			public	uint stateMask;
			public	IntPtr pszText;
			public	int cchTextMax;
			public	int iImage;
			public	int lParam;
			public	int iIndent;
		}

		[StructLayout(LayoutKind.Sequential)]
		public class HDITEM 
		{
			public int		mask; 
			public int		cxy = 0; 
			public IntPtr	pszText; 
			public IntPtr	hbm; 
			public int		cchTextMax; 
			public int		fmt; 
			public int		lParam = 0; 
			public int		iImage = 0;
			public int		iOrder = 0;
		};

		[StructLayout(LayoutKind.Sequential)]
		public struct POINT 
		{
			public int x;
			public int y;
		}

		[StructLayout(LayoutKind.Sequential)] 
		public struct LVHITTESTINFO
		{
			public POINT pt; 
			public uint  flags; 
			public int   iItem; 
			public int   iSubItem;
		}

		#endregion

		#region Win32 API method calls 

		[DllImport("user32.dll")] internal static extern int PostMessage( IntPtr hWnd, int wMsg, int wParam, int lParam );
		[DllImport("user32.dll")] internal static extern int SendMessage( IntPtr hWnd, ListViewMessage wMsg, int wParam, int lParam );
		[DllImport("user32.dll")] internal static extern int SendMessage( IntPtr hWnd, ListViewMessage wMsg, int wParam, int[] lParam );
		[DllImport("user32.dll")] internal static extern int SendMessage( IntPtr hWnd, ListViewMessage wMsg, int wParam, ref LVITEM lParam );
		[DllImport("user32.dll")] internal static extern int SendMessage( IntPtr hWnd, ListViewMessage wMsg, int wParam, ref LVHITTESTINFO lParam );
		[DllImport("user32.dll")] internal static extern int SendMessage( IntPtr hWnd, int wMsg, IntPtr wParam, HDITEM lParam );

		#endregion 

		#region Private variables 

		// The 2 bitmaps used for drawing the sort indicators in
		// the ListView column headers
		private static Bitmap imgSortAscending;
		private static Bitmap imgSortDescending;

		#endregion 

		#region Constructor 

		static ListViewHelper()
		{

			imgSortAscending = GetSortIndicator( SortOrder.Ascending );
			imgSortDescending = GetSortIndicator( SortOrder.Descending );

		}

		#endregion

		#region Public methods 

		public static void SetColumnHeaderIcon( ListView List, int ColumnIndex, Bitmap Icon, bool ImageOnRight )
		{

			int getHeader = SendMessage(
				List.Handle, 
				ListViewMessage.LVM_GETHEADER, 
				0,
				0
				);

			IntPtr hwndHeader = new IntPtr( getHeader );

			if( hwndHeader == IntPtr.Zero )
				return;
            
			HDITEM item = new HDITEM();
			item.cchTextMax = 255;

			int result = SendMessage( hwndHeader, (int) ListViewHeaderMessage.HDM_GETITEM, new IntPtr(ColumnIndex), item );
			if( result <= 0 )
				return;

			item.pszText = Marshal.StringToHGlobalAuto( List.Columns[ColumnIndex].Text );
			item.mask = (int) (HeaderIconFlags.HDI_FORMAT | HeaderIconFlags.HDI_TEXT | HeaderIconFlags.HDI_BITMAP);
			item.hbm = IntPtr.Zero;
                
			if( Icon != null ) 
			{

				item.hbm = Icon.GetHbitmap();
				item.fmt |= (int) (HeaderFormatFlags.HDF_BITMAP | HeaderFormatFlags.HDF_STRING);
				if( ImageOnRight) 
					item.fmt |= (int) HeaderFormatFlags.HDF_BITMAP_ON_RIGHT;
					
			}
			else 
			{
				item.fmt |= (int) HeaderFormatFlags.HDF_STRING;
			}

			SendMessage( hwndHeader, (int) ListViewHeaderMessage.HDM_SETITEMW, new IntPtr(ColumnIndex), item );

		}

		public static void ShowSortIndicator( ListView listView, int sortColumn, SortOrder sortOrder ) 
		{

			for( int i = 0; i < listView.Columns.Count; i++ ) 
			{

				if( i == sortColumn ) 
				{

					Bitmap bmp = null;
					if( sortOrder == SortOrder.Ascending )
						bmp = imgSortAscending;
					else if( sortOrder == SortOrder.Descending )
						bmp = imgSortDescending;

					SetColumnHeaderIcon( listView, i, bmp, true );

				}
				else 
				{
					SetColumnHeaderIcon( listView, i, null, true );
				}

			}
			
		}

		public static void SetItemIndent( ListViewItem item, int indentLevel )
		{

			if( item.ListView == null )
				throw new Exception( "The ListViewItem object passed must already be added to the ListView's Items collection" );

			int index = item.Index;

			LVITEM lvItem = new LVITEM();
			lvItem.mask = ListViewItemFlags.LVIF_INDENT;
			lvItem.iItem = index;
			lvItem.iIndent = indentLevel;

			SendMessage( item.ListView.Handle, ListViewMessage.LVM_SETITEMA, 0, ref lvItem );

		}

		public static int GetItemIndent( ListViewItem item )
		{

			if( item.ListView == null )
				throw new Exception( "The ListViewItem object passed must already be added to the ListView's Items collection" );

			int index = item.Index;

			LVITEM lvItem = new LVITEM();
			lvItem.mask = ListViewItemFlags.LVIF_INDENT;
			lvItem.iImage = index;

			SendMessage( item.ListView.Handle, ListViewMessage.LVM_GETITEMA, 0, ref lvItem );

			return lvItem.iIndent;

		}

		public static ListViewItem StateImageHitTest( ListView list, Point p )
		{

			LVHITTESTINFO htInfo;
			int ret;

			htInfo.pt.x = p.X;
			htInfo.pt.y = p.Y;
			htInfo.flags = 0;
			htInfo.iItem = 0;
			htInfo.iSubItem = 0;
			
			ret = SendMessage( 
				list.Handle,
				ListViewMessage.LVM_SUBITEMHITTEST,
				0, 
				ref htInfo
				);
			
			if( (ListViewHitTestFlags) htInfo.flags == ListViewHitTestFlags.LVHT_ONITEMSTATEICON )
			{
				return list.Items[ htInfo.iItem ];
			}
			
			return null;

		}

		#endregion 

		#region Private utility methods 

		private static Bitmap GetSortIndicator( SortOrder sortOrder )
		{

			Bitmap bitmap = new Bitmap(8, 8);

			using( Graphics canvas = Graphics.FromImage(bitmap) )
			{

				Pen lightPen = SystemPens.ControlLightLight;
				Pen shadowPen = SystemPens.ControlDark;

				canvas.FillRectangle( SystemBrushes.ControlLight, 0, 0, 8, 8 );

				if( sortOrder == SortOrder.Ascending )
				{
					canvas.DrawLine( lightPen, 0, 7, 7, 7 );
					canvas.DrawLine( lightPen, 7, 7, 4, 0 );
					canvas.DrawLine( shadowPen, 3, 0, 0, 7 );
				}
				else 
				{
					canvas.DrawLine( lightPen, 4, 7, 7, 0 );
					canvas.DrawLine( shadowPen, 3, 7, 0, 0 );
					canvas.DrawLine( shadowPen, 0, 0, 7, 0 );
				}

			}

			return bitmap;

		}

		#endregion 
		
	}

	#endregion 

}
