using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace RSSReader.Controls
{

	/// <summary>
	/// Adapter for ListViewItem class that assists in creating a 
	/// threaded ListView for RSS Items.
	/// </summary>
	public class ThreadListItem : ListViewItem
	{

		#region Public events 

		public event EventHandler ExpandedChanged;

		#endregion 

		#region Private member variables 

		private ListViewItemCollection	items;
		
		private bool	isExpanded;
		private bool	hasChildItems;
		private int		indentLevel;

		#endregion 

		#region Constructor 

		public ThreadListItem( string Text ) : base( Text )
		{

			items = new ListViewItemCollection();
			items.Clearing += new RSSReader.Controls.ListViewItemCollection.ListViewItemCollectionClearedEventHandler(items_Clearing);
			items.Inserting += new RSSReader.Controls.ListViewItemCollection.ListViewItemCollectionChangedEventHandler(items_Inserting);
			items.Removing += new RSSReader.Controls.ListViewItemCollection.ListViewItemCollectionChangedEventHandler(items_Removing);

			isExpanded = false;
			hasChildItems = false;
			indentLevel = 0;

		}

		#endregion 

		#region Public properties 

		public bool HasChildItems
		{
			get { return hasChildItems; }
			set { hasChildItems = value; }
		}

		public bool Expanded
		{
			
			get { return isExpanded; }
			set
			{
				if( value ) 
					Expand();
				else
					Collapse();
			}

		}

		public ListViewItemCollection Items
		{
			get { return items; }
		}

		internal int IndentLevel
		{
			
			get { return indentLevel; }
			set { indentLevel = value; ListViewHelper.SetItemIndent( this, value ); }

		}

		#endregion 

		#region Private utility methods 

		private void Expand()
		{

			if( isExpanded || !hasChildItems )
				return;

			int itemCount = items.Count;

			isExpanded = true;
			if( ExpandedChanged != null )
				ExpandedChanged( this, EventArgs.Empty );

			if( itemCount > 0 )
			{

				foreach( ThreadListItem item in items )
				{
				
					this.ListView.Items.Insert( this.Index + 1, item );
					item.IndentLevel = this.IndentLevel + 1;

				}

			}

		}

		private void Collapse()
		{

			if( !isExpanded || !hasChildItems )
				return;

			isExpanded = false;
			if( ExpandedChanged != null )
				ExpandedChanged( this, EventArgs.Empty );

			foreach( ListViewItem item in items )
			{
				this.ListView.Items.Remove( item );
			}

		}

		#endregion

		#region Child collection event handlers 

		private void items_Clearing(object sender)
		{
			
			foreach( ListViewItem item in items )
			{
				this.ListView.Items.Remove( item );
			}

		}

		private void items_Removing(object sender, ListViewItem value)
		{
			
			this.ListView.Items.Remove( value );

		}

		private void items_Inserting(object sender, ListViewItem value)
		{

			if( !this.isExpanded )
				return;

			if( this.ListView.Items.Count > this.Index )
			{
				
				this.ListView.Items.Insert( this.Index + 1, value );

				if( value is ThreadListItem )
					((ThreadListItem) value).IndentLevel = this.IndentLevel + 1;

			}
			else
				this.ListView.Items.Add( value );

		}

		#endregion 

	}

}
