using System;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;

using RSSReader.RSS;

namespace RSSReader
{

	#region SearchTerms class 

	/// <summary>
	/// Summary description for SearchTerms.
	/// </summary>
	public class SearchTerms
	{

		#region Private variables 

		private ArrayList	searchTerms;
		private bool		isCaseSensitive;
		private bool		matchWholeWord;
		private string		keywords;

		#endregion 

		#region Constructor 

		public SearchTerms( string Keywords, bool CaseSensitive, bool MatchWholeWord )
		{

			this.isCaseSensitive = CaseSensitive;
			this.matchWholeWord = MatchWholeWord;
			this.keywords = Keywords;

			ParseSearchTerms( Keywords );
			
		}

		#endregion 

		#region Public properties 

		public string Keywords 
		{
			get { return keywords; }
		}

		public ArrayList Terms 
		{
			get { return this.searchTerms; }
		}

		public bool IsCaseSensitive
		{
			get { return this.isCaseSensitive; }
		}

		public bool MatchWholeWord
		{
			get { return matchWholeWord; }
		}

		#endregion 

		#region Public methods 

		public int GetRank( string Data )
		{

			int rank = 0;
			foreach( SearchTerm term in this.searchTerms )
			{

				int matchCount = term.GetMatchCount( Data );
				if( matchCount != 0 )
				{
					
					if( rank == 0 )
						rank = matchCount;
					else
						rank *= matchCount;

				}

			}

			return rank;

		}

		#endregion 

		#region Private utility methods 

		private void ParseSearchTerms( string Keywords )
		{

			Tokenizer tokenizer = new Tokenizer( Keywords );

			searchTerms = new ArrayList();
			while( !tokenizer.EOF )
			{
				searchTerms.Add( new SearchTerm( tokenizer.ReadNext(), IsCaseSensitive, this.matchWholeWord ) );
			}
			
		}

		#endregion 

		#region Private classes 

		#region SearchTerm class 

		private class SearchTerm
		{

			#region Private variables 

			private string	token;
			private bool	isCaseSensitive;
			private bool	matchWholeWord;
			private Regex	searchPattern;

			#endregion 

			#region Constructor 

			public SearchTerm( string Token, bool IsCaseSensitive, bool MatchWholeWord )
			{
				
				this.token = Token;
				this.isCaseSensitive = IsCaseSensitive;
				this.matchWholeWord = MatchWholeWord;

				if( !isCaseSensitive )
					token = token.ToLower();

				if( matchWholeWord )				
					token = FormatForWholeWordMatch( token );

			}

			#endregion 

			#region Public properties 

			public string Value
			{
				get { return token; }
			}

			public bool IsCaseSensitive
			{
				get { return isCaseSensitive; }
			}

			#endregion 

			#region Public methods 

			public int GetMatchCount( string Data )
			{

				Data = StripHtml.Strip( Data );

				if( matchWholeWord )
					return GetMatchCountRegex( Data );
				else
					return GetMatchCountSubstring( Data );

			}

			#endregion 

			#region Private utility methods 

			private int GetMatchCountRegex( string Data )
			{

				if( searchPattern == null )
				{
					
					RegexOptions options = RegexOptions.Multiline | RegexOptions.Compiled;
					if( !this.isCaseSensitive )
						options |= RegexOptions.IgnoreCase;

					searchPattern = new Regex( this.token, options );

				}

				return searchPattern.Matches( Data ).Count;

			}

			private int GetMatchCountSubstring( string Data )
			{

				if( !isCaseSensitive )
					Data = Data.ToLower();

				int matchCount = 0;
				int startPos = 0;
				int length = Data.Length;

				while( startPos < length )
				{

					int nextPos = Data.IndexOf( this.token, startPos );
					if( nextPos == -1 )
						break;

					matchCount += 1;
					startPos = nextPos + token.Length;

				}

				return matchCount;

			}

			private string FormatForWholeWordMatch( string token )
			{
				return "\\b(" + token.Replace( "\\", "\\\\" ) + ")\\b";
			}

			#endregion 

		}

		#endregion 

		#region Tokenizer class 

		private class Tokenizer
		{

			#region Private variables 

			private char[]		buffer;
			private	int			pos;
			private int			tokenStart;

			#endregion 

			#region Constructor 

			public Tokenizer( string Data )
			{

				tokenStart = 0;
				pos = 0;

				// Trim existing whitespace before starting
				buffer = Data.Trim().ToCharArray();

			}

			#endregion 

			#region Public methods and properties 

			public bool EOF
			{
				get { return pos >= buffer.Length; }
			}

			public string ReadNext()
			{

				if( EOF )
					return "";
				
				ConsumeWhitespace();

				char current = Peek();
				if( current == '\'' || current == '\"' )
					return ReadQuotedString();
				else
					return ReadToken();

				
			}

			#endregion

			#region Private utility methods 

			private string ReadToken()
			{

				while( !EOF && !char.IsWhiteSpace( Peek() ) )
					Consume();

				return GetTokenValue();

			}

			private string ReadQuotedString()
			{

				// Get and consume string delimiter, since there is more than 
				//one allowed delimiter
				char delim = Peek();
				Consume();

				// Consume everything till we reach the end string delimiter
				while( !EOF && Peek() != delim )
					Consume();

				// Extract current token value
				string result = GetTokenValue().Trim( delim );

				// Get rid of trailing quoted string delimiter
				if( !EOF && Peek() == delim )
					Consume(); 

				return result;

			}

			private string GetTokenValue()
			{

				StringBuilder result = new StringBuilder();
				while( tokenStart < pos )
					result.Append( buffer[ tokenStart++ ] );

				return result.ToString();

			}

			private char Peek()
			{
				
				if( EOF )
					throw new IndexOutOfRangeException();
				else
					return buffer[ pos ];

			}

			private void Consume()
			{

				if( EOF )
					throw new IndexOutOfRangeException();

				pos += 1;

			}

			private void ConsumeWhitespace()
			{

				while( !EOF && char.IsWhiteSpace( Peek() ) )
					Consume();

				tokenStart = pos;

			}

			#endregion 

		}

		#endregion 

		#endregion 

	}

	#endregion 

	#region FeedSearchResult class 

	internal class FeedSearchResult : IComparable
	{

		#region Private variables 

		private int		matchCount;
		private object	result;

		#endregion 

		#region Constructor 

		public FeedSearchResult( int MatchCount, object SearchResult )
		{
			this.matchCount = MatchCount;
			this.result = SearchResult;
		}

		#endregion 

		#region Public properties 

		public int MatchCount
		{
			get { return this.matchCount; }
		}
		
		public object SearchResult
		{
			get { return this.result; }
		}

		#endregion 

		#region IComparable Members 

		public int CompareTo(object obj)
		{
			return this.matchCount.CompareTo( ((FeedSearchResult) obj).matchCount ) * -1;
		}

		#endregion

	}

	#endregion 

	#region FeedSearchResults class 

	public class FeedSearchResults
	{

		#region Private variables 

		private	string		keywords;
		private ArrayList	results;

		#endregion 

		#region Constructor 

		public FeedSearchResults( string Keywords, ArrayList Results )
		{
			this.keywords = Keywords;
			this.results = Results;
		}

		#endregion 

		#region Public properties 

		public string Keywords
		{
			get { return keywords; }
		}

		public ICollection Results
		{
			get { return results; }
		}

		#endregion 

		#region Public methods 

		public void Sort()
		{
			this.results.Sort();
		}

		#endregion 

	}

	#endregion 

}
